@@ -15,6 +15,7 @@ import pyatspi
 import gtk
 import os.path
 import pango
+from gobject import markup_escape_text
 from accerciser.plugin import ViewportPlugin
 from accerciser.icons import getIcon
 from accerciser.i18n import _, N_
@@ -83,7 +84,7 @@ class InterfaceViewer(ViewportPlugin):
       role_name = '%s: %s' % (role, name)
     else:
       role_name = role
-    self.label_role.set_markup('<b>%s</b>' % role_name)
+    self.label_role.set_markup('<b>%s</b>' % markup_escape_text(role_name))
     interfaces = pyatspi.listInterfaces(acc)
     for section_obj in self.sections:
       section_obj.disable()
@@ -339,7 +340,7 @@ class _SectionAccessible(_InterfaceSection):
       r_type_name = r_type_name.replace('_', ' ').lower().capitalize()
       iter = self.relations_model.append(None, [None, r_type_name, None])
       for i in range(relation.getNTargets()):
-        acc = relation.getTarget(0)
+        acc = relation.getTarget(i)
         self.relations_model.append(iter, [getIcon(acc), acc.name, acc])
     self.relations_view.expand_all()
 
@@ -420,8 +421,12 @@ class _SectionAccessible(_InterfaceSection):
     '''
     if self.node.acc == event.source:
       self.states_model.clear()
-      states = [pyatspi.stateToString(s) for s in \
-                  self.node.acc.getState().getStates()]
+      try:
+        states = [pyatspi.stateToString(s) for s in \
+                    self.node.acc.getState().getStates()]
+      except LookupError:
+        # Maybe we got a defunct state, in which case the object is diseased.
+        states = []
       states.sort()
       map(self.states_model.append, [[state] for state in states])        
 
@@ -1494,7 +1499,7 @@ class _SectionText(_InterfaceSection):
         self.text_buffer.handler_unblock(self._text_insert_handler)
            
       elif event.type.minor == 'delete':
-        call = (event.detail1, event.detail2)
+        call = (event.detail1, event.detail1 + event.detail2)
         if self.outgoing_calls['itext_delete'].isCached(call):
           return
         text_iter_end = \
